import os
import re

import axipy
from PySide2 import QtCore
from PySide2.QtCore import QFile, Signal
from PySide2.QtUiTools import QUiLoader
from PySide2.QtWidgets import QWidget, QApplication
from axipy import MapView, Layer, Point, Pnt, ListLayers
from axipy.app import Notifications

from PkkInfo.ui.tools.observer import isMapPkk
from PkkInfo.ui.tools.pkkTools import VectorPkk
from PkkInfo.ui.tools.utilHttp import FindCadastrNumber

def existInListLayers(layers,name_tab):
    for layer in layers:
        if isinstance(layer,ListLayers):
            isExist=existInListLayers(layer,name_tab)
            if isExist :
                return True
        table = layer.data_object
        if table.name == name_tab:
            return True
    return False
def layerExistMap(map,name_tab):
    for layer in map.layers:
        if isinstance(layer, ListLayers):
            isExist = existInListLayers(layer, name_tab)
            if isExist :
                return True
            else:
                continue
        table=layer.data_object
        if table.name==name_tab:
            return True
    return False
class ControlPushButtons:
    def __init__(self,list_button):
        self.__button=list_button
        self.__list_status=[]
        for i in range(len(list_button)):
            self.__list_status.append(False)
    def setEnabled(self,value):
        if not value:
            ''' запоминаем статус '''
            for i,btn in enumerate(self.__button):
                self.__list_status[i]=btn.isEnabled()
                btn.setEnabled(False)
            return
        for i,btn in enumerate(self.__button):
            btn.setEnabled(self.__list_status[i])

class DlgPkkFindObject(QWidget):

    def __init__(self,parent=None):
        self.__parentWin=parent
        self.__cls_find=None
        self.__cls_raster_vector=None
        self.__base_url_find="https://pkk.rosreestr.ru/api/features/"
        self.load_ui('PkkInfo.ui')
        self.window.ln_cad_number.textChanged.connect(self.__change_cad_number)
        self.window.pb_find.clicked.connect(self.__run_find)
        self.window.pb_get_vecor.clicked.connect(self.__run_vector)
        self.window.cmb_find_objects.currentIndexChanged.connect(self.__setInfoObject)
        self.__buttons=ControlPushButtons([self.window.pb_find,self.window.pb_get_vecor])
        jkl=0
    def load_ui(self,name_resource):
        loader = QUiLoader()
        path = os.path.join(os.path.dirname(__file__),name_resource)
        ui_file = QFile(path)
        ui_file.open(QFile.ReadOnly)
        self.window = loader.load(ui_file,self.__parentWin)
        ui_file.close()
    def __change_cad_number(self,new_val):
        ''' Проверяем корректность кадастрового номера'''
        list_token=new_val.split(':')
        if len(list_token)<3:
            '''Проверяем может это координаты '''
            tem_val=new_val.replace(" ", "")
            if tem_val.find("x=")>=0 and tem_val.find("y=")>=0:
                result=re.findall(r"[-+]?\d*\.?\d+|\d+",tem_val)
                if len(result)==2:
                    self.window.pb_find.setEnabled(True)
                    return
            self.window.pb_find.setEnabled(False)
            return
        self.window.pb_find.setEnabled(True)
        return
    def __run_find(self):
        self.window.pb_get_vecor.setEnabled(False)
        text_find=self.window.ln_cad_number.text().strip()
        if self.__cls_find is None:
            self.__cls_find=FindCadastrNumber([1,5,2,3,4,6],self.__base_url_find,self.viewNotifications)
        self.__clearResultFind()
        self.window.pb_find.setEnabled(False)
        self.__buttons.setEnabled(False)
        result=None
        ''' Проверяем вдруг это координаты '''
        tem_val=text_find.replace(" ", "")
        if tem_val.find("x=")>=0 and tem_val.find("y=")>=0:
            xy=re.findall(r"[-+]?\d*\.?\d+|\d+",tem_val)
            result=self.__cls_find.findByCoordinate(float(xy[0]),float(xy[1]))
        else:
            result=self.__cls_find.find(text_find)
        self.__buttons.setEnabled(True)
        self.window.pb_find.setEnabled(True)
        if result is not None:
            self.viewNotifications("Найден")
            self.window.pb_get_vecor.setEnabled(True)
        else:
            self.window.pb_get_vecor.setEnabled(True)
        self.__viewDataFind(result)
        jkl=0
    def __run_vector(self):
        if self.__cls_raster_vector==None:
            self.__cls_raster_vector=VectorPkk(self.viewNotifications)
        id_object=self.window.cmb_find_objects.currentIndex()
        obj_find=self.__last_result_find['features'][id_object]
        self.window.pb_get_vecor.setEnabled(False)
        self.__buttons.setEnabled(False)
        result,status=self.__cls_raster_vector.getGeometry(obj_find)
        self.__buttons.setEnabled(True)
        self.window.pb_get_vecor.setEnabled(True)

        if result is None:
            self.viewNotifications("Что то пошло не так "+status)
        else:
            #map_view_pkk=self.__findMapViewPkk()

            ''' Добавляем слой на карту и позиционируем '''
            active_mapView = axipy.gui.view_manager.active
            if isinstance(active_mapView,MapView) and isMapPkk(active_mapView.map):
                    existLayerPkkinfo=layerExistMap(active_mapView.map,self.__cls_raster_vector.nameTable)
                    if not existLayerPkkinfo:
                        ''' Добавляем в карту '''
                        layer_pkk_vect=Layer.create(self.__cls_raster_vector.table)
                        #count_layer=active_mapView.map.layers.count
                        active_mapView.map.layers.append(layer_pkk_vect)
                        count_layer_new=active_mapView.map.layers.count
                        active_mapView.map.layers.move(count_layer_new-1,0)
                    ''' смещаем карту по сентру объекта '''
                    point_centr_obj=Point(obj_find['center']['x'],obj_find['center']['y'],self.__cls_raster_vector.table.coordsystem)
                    if self.__cls_raster_vector.table.coordsystem!=active_mapView.coordsystem:
                        ''' Преобразуем центр проекцию карты '''
                        point_centr_obj=point_centr_obj.reproject(active_mapView.coordsystem)
                    #scale=active_mapView.scale
                    active_mapView.scale_with_center(1,Pnt(point_centr_obj.x,point_centr_obj.y))
            self.viewNotifications("Участок найден и добавлен во временную таблицу "+self.__cls_raster_vector.nameTable)
            Notifications.push('Получение  геометрии ',"Участок найден ("+self.window.ln_cad_number.text().strip()+") и добавлен во временную таблицу "+self.__cls_raster_vector.nameTable,Notifications.Success)
    def __viewDataFind(self,info_objects):
        self.__last_result_find=info_objects
        if self.__last_result_find is None or self.__last_result_find['total']<1:
            self.window.grp_info.setEnabled(False)
            Notifications.push('Поиск кадастрового номера',"Не найден "+self.window.ln_cad_number.text().strip(),Notifications.Critical)
            return
        self.window.grp_info.setEnabled(True)
        for ft in self.__last_result_find['features']:
            self.window.cmb_find_objects.addItem(ft['attrs']['cn'])
        self.window.cmb_find_objects.setCurrentIndex(0)
    def __findMapViewPkk(self):
        list_map=axipy.gui.view_manager.mapviews
        if list_map is None or len(list_map)==0:
            return None
        for map_view in list_map:
            if isMapPkk(map_view.map):
                return map_view
        return None
    def __clearResultFind(self):
        self.window.grp_info.setEnabled(False)
        self.window.cmb_find_objects.clear()
        self.window.ln_cad_number_ft.setText('')
        self.window.ln_type_obj.setText('')
        self.window.ln_category_type.setText('')
        self.window.p_text_adress.setText('')
    def __setInfoObject(self,id):

        ft=self.__last_result_find['features'][id]
        self.window.ln_cad_number_ft.setText(ft['attrs']['cn'])
        self.window.ln_type_obj.setText(str(ft['type']))
        try:
            self.window.ln_category_type.setText(ft['attrs']['category_type'])
        except:
            self.window.ln_category_type.setText("")
        try:
            val_address=ft['attrs']['address']
        except:
            val_address=None
        if val_address is not None:
            self.window.p_text_adress.setText(ft['attrs']['address'])
        else:
            self.window.p_text_adress.setText('')
        if 'extent' in ft:
            self.window.pb_get_vecor.setEnabled(True)
        else:
            self.window.pb_get_vecor.setEnabled(False)
    def viewNotifications(self,text):
        #Notifications.push('Поиск кадастрового номера',text,Notifications.Warning)
        self.window.lb_status.setText(text)
        import time
        time.sleep(0.0001)
        QApplication.processEvents()
        #self.window.lb_status.setText(text)
        #self.update_status.emit(text)

    def __update_status(self,text):
        Notifications.push('Поиск кадастрового номера',text,Notifications.Warning)
        import time
        time.sleep(0.0001)
        QApplication.processEvents()
    @property
    def widget(self):
        return self.window
        #return self
    def show(self):
        #self.ui.showMaximized()
        #print("Show Form")
        self.window.show()
