import os

import axipy
from PySide2.QtCore import QRectF, Qt
from PySide2.QtWidgets import QInputDialog
from axipy import MapTool, CoordSystem, Point
from axipy.app import Notifications

from PkkInfo.ui.docWidget import existByTitle
from PkkInfo.ui.tools.observer import factoryObserverPkkMap

class PkkMapTool(MapTool):
    def __init__(self):
        super(PkkMapTool, self).__init__()

        self._reset()
        self.__count_click=0
        self.__cs_4326=CoordSystem.from_epsg(4326)
    def _reset(self):
        self._start_scene_point = None
        self._end_device_point = None
    def __find_docWidget(self):
        docWidget=existByTitle(axipy.app.mainwindow.qt_object(),"Поиск объекта на публичной кадастровой карте")
        if docWidget is not None:
            if not docWidget.isVisible():
                Notifications.push('Информация с публичной кадастровой карты',"Вкладка закрыта 'Поиск объекта на публичной кадастровой карте'",Notifications.Warning)
            self.__widgetPkk=docWidget.widget()
        else:
            self.__widgetPkk=None
    def mousePressEvent(self, event):
        #print('mouse pressed')
        self.__find_docWidget()
        device_point = event.pos()
        self.__count_click=self.__count_click+1
        pos_view=self.to_scene(event.pos())
        point_4326=Point(pos_view.x,pos_view.y,self.view.coordsystem).reproject(self.__cs_4326)
        if self.__widgetPkk is not None:
            self.__widgetPkk.ln_cad_number.setText(" coordinate "+" x="+str(point_4326.x)+" y="+str(point_4326.y))
            self.__widgetPkk.pb_find.setEnabled(True)
            self.__widgetPkk.pb_find.click()
        else:
            Notifications.push('Информация с публичной кадастровой карте',"Вкладка закрыта 'Поиск объекта на публичной кадастровой карте'",Notifications.Warning)
        return self.BlockEvent
class PkkInfoMap(MapTool):
    __name='Информация с публичной кадастровой карте'
    __name_observer='MapPkk'
    __type_observer="map"
    __form_tool=None
    __docWidget=None
    __def_name_group="PKK"
    def __init__(self,parentPlugin):
        self.__parentCls=parentPlugin
        self.__initTool(parentPlugin)
        self.__docWidget=None


    def __initTool(self,parentPlugin):

        self.__observer=factoryObserverPkkMap(self.__name_observer)
        self.__observer.changed.connect(self.__changeValueObserver)
        local_file_icon=os.path.join(os.path.dirname(os.path.realpath(__file__)),'icon', 'infoTool_32x32.png')
        try:
            self.__action=parentPlugin.create_tool(self.__name,icon=local_file_icon, on_click=PkkMapTool,enable_on = self.__name_observer)
            self.__action.action.triggered.connect(self.__changeStatusButton)
        except Exception as ex:
            jkl=0
        return
    def __changeValueObserver(self):
        status=self.__observer.value()
        '''
        if status and self.__parentCls is not None:
            try:
                self.__parentCls.getPkkFindLand().initDocWidget()
            except:
                pass
        '''
    def __changeStatusButton(self,checked):
        if checked:
            self.__parentCls.getPkkFindLand().initDocWidget()
    @property
    def name(self):
        return self.__name
    @property
    def name_group(self):
        return self.__def_name_group
    @property
    def action(self):
        return self.__action
    @property
    def nameObserver(self):
        return self.__name_observer
    @property
    def typeObserver(self):
        return self.__type_observer
    @property
    def docWidget(self):
        return self.__docWidget