import tempfile

from osgeo import gdal, osr, ogr




def SetProjAndRegisrationMap(epsg:int,bound,width,height,ds):
    srs = osr.SpatialReference()
    srs.ImportFromEPSG(epsg)
    map_width=bound.Width
    map_height=bound.Height
    dx=map_width/width
    dy=map_height/height
    ds.SetProjection(srs.ExportToWkt())
    ds.SetGeoTransform([bound.Xmin, dx, 0, bound.Ymax, 0, -dy])
class RasterGdal:

    def __init__(self,source_epsg=3857):
        self.__epsg=source_epsg
    def createImage(self,params_image,image_byte):
        name=next(tempfile._get_candidate_names())
        self.__pathGdalMem='/vsimem/'+name+".png"
        gdal.FileFromMemBuffer(self.__pathGdalMem,image_byte)
        self.__memds = gdal.OpenEx(self.__pathGdalMem)

        SetProjAndRegisrationMap(self.__epsg, params_image['bound'],params_image['width'], params_image['height'], self.__memds)
        return
    def createVectort(self,resolution,path_out=None):
        path_out_vector=path_out
        if path_out is None:
            name=next(tempfile._get_candidate_names())
            path_out_vector='/vsimem/'+name+".shp"
        driver = ogr.GetDriverByName("ESRI Shapefile")
        outDatasource = driver.CreateDataSource(path_out_vector)
        srs_out = osr.SpatialReference()
        srs_out.ImportFromEPSG(3857)
        outLayer = outDatasource.CreateLayer("polygonized", srs=srs_out)
        fld = ogr.FieldDefn("val_pix", ogr.OFTInteger)
        outLayer.CreateField(fld)
        dst_field = outLayer.GetLayerDefn().GetFieldIndex("val_pix")
        gdal.Polygonize( self.__memds.GetRasterBand(1), None, outLayer, dst_field, [], callback=None )
        list_geometry=[]
        if path_out is None:
            for feature in outLayer:
                val_pix=feature.GetField('val_pix')
                if val_pix==255:
                    list_geometry.append(feature.GetGeometryRef().Clone())
        outDatasource.Destroy()
        outDatasource=None
        self.__memds = None
        gdal.Unlink(self.__pathGdalMem)
        if path_out is None:
            out_geometry=[]
            for geo in list_geometry:
                out_geometry.append(geo.Simplify(resolution))
                geo=None
            gdal.Unlink(path_out_vector)
            return out_geometry
        return None
    def saveImageToFile(self,image_byte,path_out):
        with open( path_out, "wb") as binary_file:


            binary_file.write(image_byte)






