import urllib

import axipy
from PySide2.QtCore import QPointF
from axipy import Schema, Attribute, provider_manager, Geometry, Feature, Style

from PkkInfo.ui.tools.gdalRaster import RasterGdal
from PkkInfo.ui.tools.utilHttp import smartHttpClient


class DRect:
    __p_min=None
    __p_max=None
    def __init__(self,p1,p2):
        self.__p_min=p1
        self.__p_max=p2
    @property
    def Width(self):
        return self.__p_max.x()-self.__p_min.x()
    @property
    def Height(self):
        return self.__p_max.y()-self.__p_min.y()
    @property
    def Xmin(self):
        return self.__p_min.x()
    @property
    def Ymin(self):
        return self.__p_min.y()

    @property
    def Xmax(self):
        return self.__p_max.x()
    @property
    def Ymax(self):
        return self.__p_max.y()
    @property
    def BoundParam(self):

        out_bnd="{:#.4f}".format(self.Xmin)+","+"{:#.4f}".format(self.Ymin)+","+"{:#.4f}".format(self.Xmax)+","+"{:#.4f}".format(self.Ymax)
        return out_bnd
class PkkTypeObject:
    def __init__(self):
        self.__initClasificator()
    def __initClasificator(self):
        self.__classificator={}
        '''Участки '''
        class_type_1={'filename':'arcgis/rest/services/PKK6/CadastreSelected/MapServer/export'}
        class_type_1['layer_show']='show:6,7,8,9'
        class_type_1['layerDefs']='"6":"ID = \'{id}\'","7":"ID = \'{id}\'","8":"ID = \'{id}\'","9":"ID = \'{id}\'"'
        self.__classificator['1']=class_type_1
        ''' OKC '''
        class_type_5={'filename':'arcgis/rest/services/PKK6/CadastreSelected/MapServer/export'}
        class_type_5['layer_show']='show:0,1,2,3,4,5'
        class_type_5['layerDefs']='"0":"ID = \'{id}\'","1":"ID = \'{id}\'","2":"ID = \'{id}\'","3":"ID = \'{id}\'","4":"ID = \'{id}\'","5":"ID = \'{id}\'"'
        self.__classificator['5']=class_type_5
        ''' Проекты ЗУ '''
        class_type_15={'filename':'arcgis/rest/services/PKK6/ZONESSelected/MapServer/export'}
        class_type_15['layer_show']='show:3,4,5'
        class_type_15['layerDefs']='"3":"objectid = -1","4":"ID = \'{id}\'","5":"objectid = -1"'
        self.__classificator['15']=class_type_15
        ''' Кварталы '''


        class_type_2={'filename':'arcgis/rest/services/PKK6/CadastreSelected/MapServer/export'}
        class_type_2['layer_show']='show:10,11,12'
        class_type_2['layerDefs']='"10":"ID = \'{id}\'","11":"objectid = -1","12":"objectid = -1"'
        self.__classificator['2']=class_type_2

        ''' Районы '''
        class_type_3={'filename':'arcgis/rest/services/PKK6/CadastreSelected/MapServer/export'}
        class_type_3['layer_show']='show:10,11,12'
        class_type_3['layerDefs']='"10":"objectid = -1","11":"ID = \'{id}\'","12":"objectid = -1"'
        self.__classificator['3']=class_type_3

        ''' Зоны и территории '''
        class_type_20={'filename':'arcgis/rest/services/PKK6/ZONESSelected/MapServer/export'}
        class_type_20['layer_show']='show:0,1,2,6'
        class_type_20['layerDefs']='"0":"objectid = -1","1":"objectid = -1","2":"id = \'{id}\'","6":"id = \'{id}\'"'
        self.__classificator['20']=class_type_20
        ''' Территориальные зоны'''
        class_type_6={'filename':'arcgis/rest/services/PKK6/ZONESSelected/MapServer/export'}
        class_type_6['layer_show']='show:0,1,2,6'
        class_type_6['layerDefs']='"0":"objectid = -1","1":"ID = \'{id}\'","2":"objectid = -1","6":"objectid = -1"'
        self.__classificator['6']=class_type_6
        ''' Границы '''
        class_type_7={'filename':'arcgis/rest/services/PKK6/BordersGKNSelected/MapServer/export'}
        class_type_7['layer_show']='show:0,1,5,2,6,3,7,4'
        class_type_7['layerDefs']='"0":"ID = \'{id}\'","1":"ID = \'{id}\'","2":"ID = \'{id}\'","3":"ID = \'{id}\'","4":"ID = \'{id}\'","5":"ID = \'{id}\'","6":"ID = \'{id}\'","7":"ID = \'{id}\'"'
        self.__classificator['7']=class_type_7
        ''' Округа '''
        class_type_4={'filename':'arcgis/rest/services/PKK6/CadastreSelected/MapServer/export'}
        class_type_4['layer_show']='show:10,11,12'
        class_type_4['layerDefs']='"10":"objectid = -1","11":"objectid = -1","12":"ID = \'{id}\'"'
        self.__classificator['4']=class_type_4
    def get(self,type,id):
        str_type=str(type)
        if not str_type in self.__classificator:
            return None,None,None
        name_service=self.__classificator[str_type]['filename']
        layer_show=self.__classificator[str_type]['layer_show']
        layer_def="{"+self.__classificator[str_type]['layerDefs'].format(id=id)+"}"
        return name_service,layer_show,layer_def

class PkkImage:
    def __init__(self,base_url,callback=None):
        self.__max_size=1660
        self.__base_url=base_url
        self.__client=smartHttpClient(callback)
        self.__delta_pix=5
        self.__def_layers=[6,7,8,9,10,11,12]
        self.__clasificator=PkkTypeObject()
    def getImage(self,object_cad_number):
        ft_extent=object_cad_number['extent']
        bound=DRect(QPointF(ft_extent['xmin'],ft_extent['ymin']),QPointF(ft_extent['xmax'],ft_extent['ymax']))
        new_bound=self.__calc_bound_rec(bound)
        width_image,height_image=self.__size_image(new_bound)
        cn=object_cad_number['attrs']['cn'] # кадастровый номер
        cn_id=object_cad_number['attrs']['id']
        str_layrsDef=self.__createLayerDef(cn_id)
        params_request={}
        params_request['layerDefs']=str_layrsDef
        params_request['size']=str(width_image)+","+str(height_image)
        params_request['bbox']=new_bound.BoundParam
        params_request['layers']=self.__createLayersShow()
        params_request['bboxSR']='102100'
        params_request['imageSR']='102100'
        params_request['dpi']='96'
        params_request['format']='png32'
        params_request['transparent']='true'
        params_request['f']='image'
        url_params=urllib.parse.urlencode(params_request)
        url_request=self.__base_url+"?"+url_params
        response=self.__client.get(url_request)
        if response is None or (not response.ok):
            return None,None
        if not response.headers['Content-Type']=='image/png':
            return None,None
        params_image={}
        params_image['width']=width_image
        params_image['height']=height_image
        params_image['bound']=new_bound
        return response.content,params_image
    def getImageNew(self,object_cad_number):
        ft_extent=object_cad_number['extent']
        bound=DRect(QPointF(ft_extent['xmin'],ft_extent['ymin']),QPointF(ft_extent['xmax'],ft_extent['ymax']))
        new_bound=self.__calc_bound_rec(bound)
        width_image,height_image=self.__size_image(new_bound)
        #cn=object_cad_number['attrs']['cn'] # кадастровый номер
        cn_id=object_cad_number['attrs']['id']
        service_name,layerShow,str_layrsDef=self.__clasificator.get(object_cad_number['type'],cn_id)
        if service_name is None:
            return None,"не поддерживаемый тип"
        curent_url=self.__base_url+service_name
        #str_layrsDef=self.__createLayerDef(cn_id)
        params_request={}
        params_request['layerDefs']=str_layrsDef
        params_request['size']=str(width_image)+","+str(height_image)
        params_request['bbox']=new_bound.BoundParam
        params_request['layers']=layerShow
        params_request['bboxSR']='102100'
        params_request['imageSR']='102100'
        params_request['dpi']='96'
        params_request['format']='png32'
        params_request['transparent']='true'
        params_request['f']='image'
        url_params=urllib.parse.urlencode(params_request)
        url_request=curent_url+"?"+url_params
        response=self.__client.get(url_request)
        if response is None or (not response.ok):
            return None,None
        if not response.headers['Content-Type']=='image/png':
            return None,None
        params_image={}
        params_image['width']=width_image
        params_image['height']=height_image
        params_image['bound']=new_bound
        return response.content,params_image
    def __calc_bound_rec(self,bound:DRect):
        width=bound.Width
        height=bound.Height
        max_side=width
        if width<height:
            max_side=height
        resolution=max_side/96
        resolution=max_side/self.__max_size
        bound_delta=self.__delta_pix*resolution
        new_xmin=bound.Xmin-bound_delta
        new_ymin=bound.Ymin-bound_delta
        new_xmax=bound.Xmax+bound_delta
        new_ymax=bound.Ymax+bound_delta

        return DRect(QPointF(new_xmin,new_ymin),QPointF(new_xmax,new_ymax))
    def __size_image(self,bound:DRect):
        if bound.Width>bound.Height:
            resolution=bound.Width/self.__max_size
            coef=bound.Height/bound.Width
            width=self.__max_size
            height=int(bound.Height/resolution+0.5)
            return width,height
        resolution=bound.Height/self.__max_size
        coef=bound.Height/bound.Width
        height=self.__max_size
        width=int(bound.Width/resolution+0.5)
        return width,height
    def __createLayerDef(self,cad_munber):
        out_str='{'
        cnt_layer=len(self.__def_layers)-1
        for i,layer_id in enumerate(self.__def_layers):
            out_str=out_str+'"'+str(layer_id)+'":"ID='+"'"+cad_munber+"'"+'"'
            if i!=cnt_layer:
                out_str=out_str+","
        out_str=out_str+"}"
        return out_str
    def __createLayersShow(self):
        out_="show:"+str(self.__def_layers[0])
        for i in range(1,len(self.__def_layers)):
            out_=out_+","
            out_=out_+str(self.__def_layers[i])
        return out_
class VectorPkk:
    def __init__(self,callback=None):
        self.__name_mem="PKK_участки"
        #self.__base_url='https://pkk.rosreestr.ru/arcgis/rest/services/PKK6/CadastreSelected/MapServer/export'
        self.__base_url='https://pkk.rosreestr.ru/'
        self.__pkk_image=PkkImage(self.__base_url,callback)
        self.__raster=RasterGdal()
        self.__initTempTable()
        self.__def_style= Style.from_mapinfo("Pen (2, 2, 8388608) Brush (2, 16777072, 16777215)")
    def __initTempTable(self):
        self.__curentTab=axipy.data_manager.find(self.__name_mem)
        if self.__curentTab is None:
            self.__curentTab=self.__createTempTab()
    def __createTempTab(self):
        schema = Schema(
            Attribute.integer('type'),
            Attribute.string('cn',32),
            Attribute.string('category_type',12),
            Attribute.string('address',255),
            Attribute.float('точность'),
            coordsystem='epsg:3857'
        )
        self.__name_mem="PKK_участки"
        definition = {
            'src': '',
            'dataobject': self.__name_mem,
            'schema':schema}
        mem_table = provider_manager.create(definition)
        return mem_table
    def  getGeometry(self,pkk_json):

        #obj_image,param_image=self.__pkk_image.getImage(pkk_json)
        obj_image,param_image=self.__pkk_image.getImageNew(pkk_json)
        if obj_image is None:
            return None,"Ошибка получения"
        tab_temp=axipy.data_manager.find(self.__name_mem)
        if tab_temp is None:
            self.__curentTab=self.__createTempTab()
        self.__raster.createImage(param_image,obj_image)
        ''' Получаем вектор из растра'''
        resolution=param_image['bound'].Width/param_image['width']
        gdal_geometrys=self.__raster.createVectort(resolution*1.4,None)
        if gdal_geometrys is None or len(gdal_geometrys)==0:
            return None,"Нет геометрии"
        for geo_gdal in gdal_geometrys:
            axi_geo=Geometry.from_wkb(geo_gdal.ExportToWkb(),self.__curentTab.coordsystem)
            values={}
            values['type']=pkk_json['type']
            if 'cn' in pkk_json['attrs']:
                values['cn']=pkk_json['attrs']['cn']
            if 'category_type' in pkk_json['attrs']:
                values['category_type']=pkk_json['attrs']['category_type']
            if 'address' in pkk_json['attrs']:
                values['address']=pkk_json['attrs']['address']
            values['точность']=resolution*1.4
            ft=Feature(values,geometry=axi_geo,style=self.__def_style)

            self.__curentTab.insert([ft])
            geo_gdal=None
        return True,None
    @property
    def nameTable(self):
        return self.__name_mem
    @property
    def table(self):
        return self.__curentTab




